  // Uses gamma-ray (and eventually also other products) hadronic production matrices from
  // Huang, C.-Y.; Park, S.-E.; Pohl, M.; Daniels, C. D. 2007 Astroparticle Physics, Volume 27, Issue 5, p. 429-439.
  // http://cdsads.u-strasbg.fr/cgi-bin/bib_query?2007APh....27..429H
  // More detail in Delahaye et al. 2011 http://adsabs.harvard.edu/abs/2011A%26A...531A..37D

  // Data can be downloaded from http://www.app.physik.uni-potsdam.de/gamma-prod.html
  // data files contain matrices with 201 lines of 374 entries, each entry is a 15 byte number in format .1234567890D-01
  // each of the 201 lines is for one gamma-ray (or other product) energy, 
  // with the entries corresponding to the 374 proton or Helium energies
  // energies are total in GeV/nucleon for proton and Helium, kinetic energy in GeV for products.

  // Implemented here so far:
  // CR Protons + ISM -> gamma rays
  // CR Helium  + ISM -> gamma rays

  // other secondaries will follow later 

// Requires the following files:
// source:
// CR_Spectrum.cc  CR_Spectrum.h  ProductionMatrices.cc  ProductionMatrices.h  test_ProductionMatrices.cc
// data:
// espectra_gamma.decay.he.matrix.data  espectra_gamma.decay.p.matrix.data  prodxsection.he.matrix.data  prodxsection.p.matrix.data

 // Sample program is in test_ProductionMatrices.cc
//  to compile and run sample program:
//  g++ *cc -O3 (also tested with intel icpc)
//  ./a.out  (lots of screen output)
//  to see just the sample output:
// ./a.out | grep sample

// Version 1.0  13 Apr 2012
// Andy Strong, MPE, Garching, aws@mpe.mpg.de
// Available from http://www.mpe.mpg.de/~aws/propagate.html
 

//////////////////////////////////////////////////////////////////////////////////////////////////

using namespace std;
#include<iostream>
#include<fstream>
#include<cstring>
#include<cstdlib>
#include<vector>
#include<valarray>

#include"ProductionMatrices.h"
#include"CR_Spectrum.h"


/////////////////////////////////////////////////
int  ProductionMatrices::init(string energy_type_,string energy_units_)
{
  cout<<">>ProductionMatrices::init";
  energy_type =energy_type_;
  energy_units=energy_units_;

  cout<<" energy_type = " <<energy_type;
  cout<<" energy_units = "<<energy_units<<endl;

  if (energy_type !="total_energy"&&energy_type !="kinetic_energy"){ cout<<"invalid energy type !"<<endl; exit(1);}
  if (energy_units!="GeV"         &&energy_units!="MeV"           ){ cout<<"invalid energy units!"<<endl; exit(1);}

  cout<<"<<ProductionMatrices::init"<<endl;
  return 0;
}
/////////////////////////////////////////////////////

int ProductionMatrices::read()
{




  int debug=0;

  char infile[200];

  int process;

  // read the gamma-ray spectra matrices

 for(process=1;process<=2;process++)
 {

  if(process==1)strcpy(infile,"espectra_gamma.decay.p.matrix.data");
  if(process==2)strcpy(infile,"espectra_gamma.decay.he.matrix.data");

  cout<<"reading from "<<infile<<endl;

  ifstream in;
  in.open(infile);

  char *buffer;



  nE_proton=374;
  nE_gamma =201;

  int number_size=15;

  double *values;
  values = new double[nE_proton];

  vector<valarray<double> > matrix;
  matrix.resize(nE_proton);
  for(int i=0;i<nE_proton;i++) matrix[i].resize(nE_gamma);

 // see Schildt c++ Complete Reference page 554: note the +1 needed for getline

  int linelength=nE_proton*number_size;
  buffer =new char[linelength];


 for(int iE_gamma=0;iE_gamma<nE_gamma;iE_gamma++)
 {

  in.getline(buffer,linelength+1);
 
  if(debug==1)cout<<"input buffer="<<buffer<<endl;

  // change number format from fortran D to c++ e

  for(int i=0;i<nE_proton;i++)
  {
  //cout<<"strchr:"<<strchr(buffer,'D')<<endl; //"D" does not compile, 'D' does

   strncpy(strchr(buffer,'D'),"e",1); // copies one character

  }

  if(debug==1)  cout<<"modified buffer="<<buffer<<endl;

  for(int i=0;i<nE_proton;i++)
  {  
   sscanf(&buffer[i*number_size],"%le",&values[i]); // le = double, Schildt p.209
  }

  if(debug==1)
  {
   cout<<"values=";
   for(int i=0;i<nE_proton;i++)cout<<values[i]<<" "; cout<<endl;
  }


  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)
   matrix[iE_proton][iE_gamma] =   values[iE_proton];

    }//iE_gamma

 in.close();


 E_proton.resize(nE_proton);
 E_gamma .resize(nE_gamma );

 // equation 1 of Huang manual note-1.pdf
 // TOTAL energy in GeV/nucleon
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)
    E_proton[iE_proton] = 1.24 * pow( (1.0+0.05), iE_proton-1);

 // equation 2 of Huang manual note-1.pdf
  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)
    E_gamma[iE_gamma]= exp(log(0.01) + iE_gamma*(log(1.e8)-log(0.01))/nE_gamma);


 if(debug==2)
 {

  cout<<"proton total energies (GeV/nucleon)="<<endl;
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)cout<< E_proton[iE_proton]<<" "; cout<<endl;

  cout<<"gamma  energies (GeV)="<<endl;
  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)cout<< E_gamma [iE_gamma ]<<" "; cout<<endl;




  cout<<"matrix="<<endl;

  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)
  {
   cout<<"iE_gamma="<<iE_gamma<<endl;

  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)
         cout<<matrix[iE_proton][iE_gamma]<<" ";
    
  cout<<endl;
  }

 }//debug==2


 if(process==1) gamma_decay_p_matrix =matrix;
 if(process==2) gamma_decay_he_matrix=matrix;


 }// process


 ///////////////////////////////////////////////////////////////////////

  // read the total inelastic cross-sections

 debug=1;

 prodxsection_p_matrix .resize(nE_proton);
 prodxsection_he_matrix.resize(nE_proton);

 for(process=1;process<=2;process++)
 {

  if(process==1)strcpy(infile,"prodxsection.p.matrix.data");
  if(process==2)strcpy(infile,"prodxsection.he.matrix.data");

  cout<<"reading from "<<infile<<endl;

  ifstream in;
  in.open(infile);

  char *buffer;
  

  int number_size=15;

  double *values;
  values = new double[nE_proton];

  vector<valarray<double> > matrix;
  matrix.resize(nE_proton);
  for(int i=0;i<nE_proton;i++) matrix[i].resize(1);

 // see Schildt c++ Complete Reference page 554: note the +1 needed for getline

  int linelength=nE_proton*number_size;
  buffer =new char[linelength];

  in.getline(buffer,linelength+1);
 
  if(debug==1)cout<<"prodxsection input buffer="<<buffer<<endl;

  // change number format from fortran D to c++ e

  for(int i=0;i<nE_proton;i++)
  {
  //cout<<"strchr:"<<strchr(buffer,'D')<<endl; //"D" does not compile, 'D' does

   strncpy(strchr(buffer,'D'),"e",1); // copies one character

  }

  if(debug==1)  cout<<" prodx modified buffer="<<buffer<<endl;

  for(int i=0;i<nE_proton;i++)
  {  
   sscanf(&buffer[i*number_size],"%le",&values[i]); // le = double, Schildt p.209
  }

  if(debug==1)
  {
   cout<<"prodx values=";
   for(int i=0;i<nE_proton;i++)cout<<values[i]<<" "; cout<<endl;
  }


 in.close();


 if(debug==2)
 {

  cout<<"proton total energies (GeV/nucleon)="<<endl;
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)cout<< E_proton[iE_proton]<<" "; cout<<endl;

  cout<<"prodx values="<<endl;

  
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)  cout<<values[iE_proton]<<" ";
    
  cout<<endl;
  

 }//debug==2


 if(process==1) for(int iE_proton=0;iE_proton<nE_proton;iE_proton++) prodxsection_p_matrix [iE_proton]=values[iE_proton];
 if(process==2) for(int iE_proton=0;iE_proton<nE_proton;iE_proton++) prodxsection_he_matrix[iE_proton]=values[iE_proton];


 }// process

 matrices_read=1;

  return 0;
};

/////////////////////////////////////////////////////

int ProductionMatrices::print()
{

  cout<<"ProductionMatrices::print"<<endl;

  // print input matrices

  cout<<"proton total energies (GeV/nucleon)="<<endl;
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)cout<< E_proton[iE_proton]<<" "; cout<<endl;

  cout<<"gamma  energies (GeV)="<<endl;
  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)cout<< E_gamma [iE_gamma ]<<" "; cout<<endl;

  cout<<" input prodxsection_p_matrix  = ";
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++) cout<< prodxsection_p_matrix  [iE_proton]<<" "; cout<<endl;

  cout<<" input prodxsection_he_matrix  = ";
  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++) cout<< prodxsection_he_matrix [iE_proton]<<" "; cout<<endl;

  cout<<" input gamma_decay_p_matrix="<<endl;

  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)
  {
   cout<<"iE_gamma="<<iE_gamma<<" E_gamma="<<E_gamma[iE_gamma] <<endl;

  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)
         cout<<  gamma_decay_p_matrix[iE_proton][iE_gamma]<<" ";
    
  cout<<endl;
  }

  cout<<" input gamma_decay_he_matrix="<<endl;

  for(int iE_gamma =0;iE_gamma <nE_gamma ;iE_gamma ++)
  {
    cout<<"iE_gamma="<<iE_gamma<<" E_gamma="<<E_gamma[iE_gamma]<<endl;

  for(int iE_proton=0;iE_proton<nE_proton;iE_proton++)
         cout<<  gamma_decay_he_matrix[iE_proton][iE_gamma]<<" ";
    
  cout<<endl;
  }

  // print rebinned matrices
  cout<<endl;
  cout<<"rebinned proton total energies ("<<energy_units<<"/nucleon)="<<endl;
  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)cout<< E_proton_rebin[iE_proton_rebin]<<" "; cout<<endl;

  cout<<endl;
  cout<<"rebinned gamma  energies ("<<energy_units<<")="<<endl;
  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)cout<< E_gamma_rebin [iE_gamma_rebin ]<<" "; cout<<endl;

  cout<<endl;
  cout<<" view gamma_decay_p_matrix_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
  {
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]<<" "<<energy_units<<endl;

  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
         cout<<  gamma_decay_p_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]<<" ";
    
  cout<<endl;
  }
  cout<<endl;
  cout<<" view gamma_decay_he_matrix_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
  {
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]<<" "<<energy_units<<endl;

  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
         cout<<  gamma_decay_he_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]<<" ";
    
  cout<<endl;
  }


  // print emissivities

  cout<<"  gamma_decay_p_emissivity_rebin="<<endl;
  cout<<"  energy units="<<energy_units<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]
        <<" "<<energy_units<<" proton emissivity= "<< gamma_decay_p_emissivity_rebin[iE_gamma_rebin]
	<<" sr-1 s-1 "<<energy_units<<"-1 H atom-1 "
        << " *E^2: "<< gamma_decay_p_emissivity_rebin[iE_gamma_rebin]*pow(E_gamma_rebin[iE_gamma_rebin],2)*1e3
        <<" MeV sr-1 s-1  H atom-1 "
        <<endl;

  cout<<"  gamma_decay_he_emissivity_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]
        <<" "<<energy_units<<"  helium emissivity= "<< gamma_decay_he_emissivity_rebin[iE_gamma_rebin]
	<<" sr-1 s-1 "<<energy_units<<"-1 H atom-1 "
        << " *E^2: "<< gamma_decay_he_emissivity_rebin[iE_gamma_rebin]*pow(E_gamma_rebin[iE_gamma_rebin],2)*1e3
        <<" MeV sr-1 s-1  H atom-1 "
        <<endl;

  cout<<"   gamma_decay_p_emissivity_rebin + gamma_decay_he_emissivity_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]
        <<" GeV proton + helium emissivity= "
        <<  gamma_decay_p_emissivity_rebin[iE_gamma_rebin]+gamma_decay_he_emissivity_rebin[iE_gamma_rebin]
	<<" sr-1 s-1 GeV-1 H atom-1"
        << " *E^2: "
        <<  (gamma_decay_p_emissivity_rebin [iE_gamma_rebin]
	    +gamma_decay_he_emissivity_rebin[iE_gamma_rebin])*pow(E_gamma_rebin[iE_gamma_rebin],2)*1e3
        <<" MeV sr-1 s-1  H atom-1 "
        <<endl;


  return 0;
};

//////////////////////////////////////////////////////////////////

int ProductionMatrices::rebin(valarray<double> E_proton_rebin_, valarray<double> E_gamma_rebin_ )
{
 cout<<">>ProductionMatrices::rebin"<<endl;


 int debug=0;
 // if(energy_units_=="MeV")debug=1;

 if(matrices_read!= 1) {cout<<" ProductionMatrices:: rebin: matrices have not been not read !"<<endl; exit(1);}

  nE_proton_rebin=E_proton_rebin_.size();
  nE_gamma_rebin = E_gamma_rebin_.size();

  E_proton_rebin.resize(nE_proton_rebin);
  E_gamma_rebin .resize(nE_gamma_rebin );
 
  E_proton_rebin= E_proton_rebin_;
  E_gamma_rebin = E_gamma_rebin_ ;

  ////  energy_units = energy_units_; // now done in init ! AWS20110411

  cout<<"ProductionMatrices::  rebin: energy_units ="<<energy_units<<endl;

  if(energy_units != "GeV" && energy_units != "MeV")
    {cout<<"ProductionMatrices::rebin: energy_units=" <<energy_units<< " must be GeV or MeV !"<<endl; exit(-1);}

  cout<<"ProductionMatrices::rebin nE_proton_rebin="<< nE_proton_rebin<<endl;
  cout<<"ProductionMatrices::rebin nE_gamma_rebin ="<< nE_gamma_rebin <<endl;

  cout<<"ProductionMatrices::rebin E_proton_rebin ["<<energy_units<<"] =";
  for(int iE_proton_rebin=0; iE_proton_rebin<nE_proton_rebin; iE_proton_rebin++)cout<<E_proton_rebin[ iE_proton_rebin]<<" ";
  cout<<endl;

  cout<<"ProductionMatrices::rebin E_gamma_rebin ["<<energy_units<<"] =";
  for(int iE_gamma_rebin=0;  iE_gamma_rebin <nE_gamma_rebin;  iE_gamma_rebin++) cout<<E_gamma_rebin  [ iE_gamma_rebin]<<" ";
  cout<<endl;

  // working input matrix
  vector<valarray<double> > matrix;

  // working input inelastic cross-section
  valarray<double> prodx;
  prodx.resize(nE_proton);

  //working rebinned matrix
  vector<valarray<double> > matrix_rebin;
  matrix_rebin.resize(nE_proton_rebin);
  for(int i=0;i<nE_proton_rebin;i++) matrix_rebin[i].resize(nE_gamma_rebin);

  gamma_decay_p_matrix_rebin .clear(); // in case called multiple times, for "=" to work
  gamma_decay_he_matrix_rebin.clear(); // in case called multiple times, for "=" to work
  
int process;

  for(process=1;process<=2;process++)
  {
    if(process==1) prodx  =  prodxsection_p_matrix;
    if(process==2) prodx  =  prodxsection_he_matrix;

    if(process==1) matrix =  gamma_decay_p_matrix;
    if(process==2) matrix =  gamma_decay_he_matrix;

  for(int i=0;i<nE_proton_rebin;i++) matrix_rebin[i]=0.;
  

  // using formula from Huang etal. note.pdf (see above) in GeV
  //   E_proton[iE_proton] = 1.24 * pow( (1.0+0.05), iE_proton-1);
  // log(E_proton[iE_proton]) = log(1.24)  +log(1.0+0.05)* (iE_proton-1);

  //     E_gamma[iE_gamma]= exp(log(0.01) + iE_gamma*(log(1.e8)-log(0.01))/nE_gamma);
  //log( E_gamma[iE_gamma])=    log(0.01) + iE_gamma*(log(1.e8)-log(0.01))/nE_gamma);

  // the rebinning is either on GeV or MeV scale, while original is GeV, so compute indices accordingly




  for(int iE_proton_rebin=0; iE_proton_rebin<nE_proton_rebin; iE_proton_rebin++)
  {

    double E_proton_total_rebin_GeV; // since original matrices are in GeV and total energy/nucleon

    if(energy_units=="GeV") E_proton_total_rebin_GeV = E_proton_rebin[ iE_proton_rebin];
    if(energy_units=="MeV") E_proton_total_rebin_GeV = E_proton_rebin[ iE_proton_rebin]* 1.e-3; // MeV -> GeV

    double m_p   = 0.938 ;      // proton mass in GeV  (deprecated hard coding of physics constant !)
    if(energy_type=="kinetic_energy")  E_proton_total_rebin_GeV += m_p; // kinetic energy/nucleon-> total energy/nucleon

 
    if ( E_proton_total_rebin_GeV > m_p) // catch case where total energy is less than rest mass
    {

     int iE_proton;


     iE_proton=(log(E_proton_total_rebin_GeV)-log(1.24))/log(1.05) +1; // rebin in GeV or MeV

     if(debug==1) cout<<"E_proton_rebin="<<E_proton_rebin[ iE_proton_rebin]<<" "<<energy_units<<" iE_proton="
		      << iE_proton<<" E_proton="<<E_proton[iE_proton]<<" - "<<E_proton[iE_proton+1] << " GeV"<<endl;

    if(iE_proton>=0 && iE_proton<nE_proton-1)
    {
     for(int iE_gamma_rebin =0; iE_gamma_rebin <nE_gamma_rebin ; iE_gamma_rebin++ )
     {

      double E_gamma_rebin_GeV; // since original matrices are in GeV
      if(energy_units=="GeV") E_gamma_rebin_GeV=E_gamma_rebin[iE_gamma_rebin];
      if(energy_units=="MeV") E_gamma_rebin_GeV=E_gamma_rebin[iE_gamma_rebin]*1.e-3;

      int iE_gamma;
 

       iE_gamma=(log(  E_gamma_rebin_GeV     )- log(0.01))/((log(1.e8)-log(0.01))/nE_gamma); // rebin in GeV or MeV

      if(debug==1) cout<<"E_gamma_rebin="<<E_gamma_rebin[ iE_gamma_rebin]<<" "<<energy_units<<" iE_gamma ="
		       << iE_gamma <<" E_gamma ="<<E_gamma [iE_gamma ]<<" - "<<E_gamma [iE_gamma+1]<<" GeV" <<endl;


      double v; 


     if(iE_gamma>=0 && iE_gamma<nE_gamma-1)
     {
	double v1=prodx[iE_proton  ] * matrix[iE_proton  ][iE_gamma];
	double v2=prodx[iE_proton+1] * matrix[iE_proton+1][iE_gamma];

	double v3=prodx[iE_proton  ] * matrix[iE_proton  ][iE_gamma+1];
	double v4=prodx[iE_proton+1] * matrix[iE_proton+1][iE_gamma+1];

 
	// first interpolate to E_proton_rebin for each E_gamma
        // differential cross-section (mb GeV-1)  = total inelastic cross-section (mb) * differential spectrum (GeV-1)

        double v12= v1+
                   (v2-v1)
                  *(E_proton_total_rebin_GeV              - E_proton[ iE_proton])           //AWS20120411
                  /(E_proton      [iE_proton+1]           - E_proton[ iE_proton]);



        double v34= v3+
                   (v4-v3)
                  *(E_proton_total_rebin_GeV              - E_proton[ iE_proton])           //AWS20120411
                  /(E_proton      [iE_proton+1]           - E_proton[ iE_proton]);

        
        if(debug==1) cout<<"v1="<<v1<<" v2="<<v2<<" v12="<<v12<<endl;
        if(debug==1) cout<<"v3="<<v3<<" v4="<<v4<<" v34="<<v34<<endl;


	// then interpolate to E_gamma_rebin

               v=  v12+
	          (v34-v12)
                  *(E_gamma_rebin_GeV                   - E_gamma [ iE_gamma ])
                  /(E_gamma      [iE_gamma +1]          - E_gamma [ iE_gamma ]);

        if(debug==1) cout<<"v before gamma energy conversion ="<<v<<endl;

        if (energy_units=="MeV") v*=1.e-3;// gammas GeV-1 -> MeV-1 

        if(debug==1) cout<<"v after gamma energy conversion ="<<v<<endl;

        matrix_rebin[iE_proton_rebin][iE_gamma_rebin] = v;





     }//if  iE_gamma_rebin

     }// iE_gamma_rebin

    }//if  iE_proton_rebin

   } // if total energy>mp

  }// iE_proton_rebin




    if(debug==2)
    {
     cout<<" rebin: debug of               matrix_rebin="<<endl;

     for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
     {
      cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]<<endl;

      for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
         cout<<                matrix_rebin[iE_proton_rebin][iE_gamma_rebin]<<" ";
    
      cout<<endl;
     }
    } 

  if(process==1)gamma_decay_p_matrix_rebin = matrix_rebin;// NB works right for multiple calls only if clear() called
  if(process==2)gamma_decay_he_matrix_rebin= matrix_rebin;// NB works right for multiple calls only if clear() called

  }// process


 if(debug==2)
    {
  cout<<" rebin: debug of gamma_decay_p_matrix_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
  {
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]<<endl;

  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
         cout<<  gamma_decay_p_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]<<" ";
    
  cout<<endl;
  }

  cout<<" rebin: debug of gamma_decay_he_matrix_rebin="<<endl;

  for(int iE_gamma_rebin =0;iE_gamma_rebin <nE_gamma_rebin ;iE_gamma_rebin ++)
  {
    cout<<"iE_gamma_rebin="<<iE_gamma_rebin<<" E_gamma_rebin="<<E_gamma_rebin[iE_gamma_rebin]<<endl;

  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
         cout<<  gamma_decay_he_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]<<" ";
    
  cout<<endl;
  }
    }


  matrices_rebinned=1;

 cout<<"<<ProductionMatrices::rebin"<<endl;

  return 0;
};

///////////////////////////////////////////////////////////////////

  int ProductionMatrices::emissivity(valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin )
{
  cout<<"ProductionMatrices::emissivity    "<<endl;

  // proton and helium spectra are assumed to be fluxes cm-2 sr-1 s-1 GeV-1
  // (so velocity term is already included)
  // giving emissivity sr-1 s-1 GeV-1 H atom-1

  if(matrices_rebinned!= 1) {cout<<" ProductionMatrices:: emissivity: matrices have not been not rebinned!"<<endl; exit(1);}

  gamma_decay_p_emissivity_rebin .resize(nE_gamma_rebin);//AWS20120328
  gamma_decay_he_emissivity_rebin.resize(nE_gamma_rebin);//AWS20120328

  gamma_decay_p_emissivity_rebin =0.;
  gamma_decay_he_emissivity_rebin=0.;

  int debug=0;

  double m_p   = 0.938 ;      // proton mass in GeV  (deprecated hard coding of physics constant !)
  double c     = 3.0e10;      // velocity of light   (deprecated hard coding  of physics constant!)
  
  // it is assumed the spectrum is on a log energy scale, hence multiply by E_proton and apply factor 

for (int iE_gamma_rebin =0; iE_gamma_rebin <nE_gamma_rebin ; iE_gamma_rebin++ )
  {
   for(int iE_proton_rebin=0; iE_proton_rebin<nE_proton_rebin; iE_proton_rebin++)
   {
     double g    = E_proton_rebin[iE_proton_rebin]/m_p;// Lorentz factor of proton
     double beta = sqrt(1.0-1.0/g/g);                  // velocity/c     of proton

     // energy is total energy per nucleon in GeV
     // since energy is per nucleon, protons and helium have same velocity
     // but velocity not needed if spectra are already fluxes. Keep for case of CR densities e.g. for GALPROP

    gamma_decay_p_emissivity_rebin[ iE_gamma_rebin]+= 
                                                        proton_spectrum_rebin     [iE_proton_rebin]
                                                      * gamma_decay_p_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]
                                                      * E_proton_rebin[iE_proton_rebin];

    gamma_decay_he_emissivity_rebin[ iE_gamma_rebin]+=  
                                                        helium_spectrum_rebin      [iE_proton_rebin]
                                                      * gamma_decay_he_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]
                                                      * E_proton_rebin[iE_proton_rebin];

 
    if(debug==1)
      cout<<"emissivity::test iE_gamma_rebin="<<iE_gamma_rebin  <<" E_gamma_rebin= "<< E_gamma_rebin [iE_gamma_rebin]
        <<" E_proton_rebin= "   << E_proton_rebin[iE_proton_rebin]
        <<" proton spectrum="<<proton_spectrum_rebin     [iE_proton_rebin]
	<<" matrix="<<gamma_decay_p_matrix_rebin[iE_proton_rebin][iE_gamma_rebin]
        <<" gamma_decay_p_emissivity_rebin= "<<gamma_decay_p_emissivity_rebin[ iE_gamma_rebin]
	<<" energy_units="<<energy_units
        <<endl;

  

    }
    }

  gamma_decay_p_emissivity_rebin  *=  log(E_proton_rebin[1]/E_proton_rebin[0]); // since log scale integration
  gamma_decay_he_emissivity_rebin *=  log(E_proton_rebin[1]/E_proton_rebin[0]); // since log scale integration



  double integral_proton_flux=0.;
  double integral_helium_flux=0.;

  for(int iE_proton_rebin=0; iE_proton_rebin<nE_proton_rebin; iE_proton_rebin++)
  {
   integral_proton_flux                          += 
                                                        proton_spectrum_rebin    [iE_proton_rebin]
                                                      * E_proton_rebin[iE_proton_rebin];

    integral_helium_flux                          += 
                                                        helium_spectrum_rebin    [iE_proton_rebin]
                                                      * E_proton_rebin[iE_proton_rebin];
}


  integral_proton_flux  *=  log(E_proton_rebin[1]/E_proton_rebin[0]);
  integral_helium_flux  *=  log(E_proton_rebin[1]/E_proton_rebin[0]);

  if(debug==1) cout<<"factor for log integration="<< log(E_proton_rebin[1]/E_proton_rebin[0])<<endl;

  gamma_decay_p_emissivity_rebin  *= 1.0e-27; // cross-section is in millibarn GeV-1
  gamma_decay_he_emissivity_rebin *= 1.0e-27; // cross-section is in millibarn GeV-1


  double H_fraction=0.9;  // Huang etal is per atom of ISM with H fraction 0.9. Convert to per H atom standard
  gamma_decay_p_emissivity_rebin /= H_fraction;
  gamma_decay_he_emissivity_rebin/= H_fraction;

  if(debug==1) cout<<"integral proton flux="<<integral_proton_flux<<" cm-2 sr-1 s-1"<<endl;
  if(debug==1) cout<<"integral helium flux="<<integral_helium_flux<<" cm-2 sr-1 s-1"<<endl;

  return 0;
}

///////////////////////////////////////////////////////////////////

double  ProductionMatrices::emissivity_band
          (valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin,
           valarray<double> E_proton_rebin_,
           double E_gamma_band_min, double E_gamma_band_max, double E_gamma_factor)
{

  int debug=1;

  if(debug==1)cout<<"ProductionMatrices::emissivity_band"<<endl;

  valarray<double> E_gamma_rebin_work;
  

  int nE_gamma_rebin_=log(E_gamma_band_max/E_gamma_band_min)/log(E_gamma_factor)+1.001; // redone in rebin ? check

  E_gamma_rebin_work.resize(nE_gamma_rebin_);

  for(int iE_gamma_rebin=0;iE_gamma_rebin<nE_gamma_rebin_;iE_gamma_rebin++)
        E_gamma_rebin_work[iE_gamma_rebin] = E_gamma_band_min * pow(E_gamma_factor,iE_gamma_rebin);

  

  rebin(E_proton_rebin_,E_gamma_rebin_work);

  emissivity(proton_spectrum_rebin, helium_spectrum_rebin);

  double emiss_band=0;//zero'd AWS20120327

  for(int iE_gamma_rebin=0;iE_gamma_rebin<nE_gamma_rebin;iE_gamma_rebin++)
    {

      emiss_band+= gamma_decay_p_emissivity_rebin  [iE_gamma_rebin]*E_gamma_rebin[iE_gamma_rebin];
      emiss_band+= gamma_decay_he_emissivity_rebin [iE_gamma_rebin]*E_gamma_rebin[iE_gamma_rebin];

      if(debug==1)cout<<"emissivity_band: E_gamma= "<<E_gamma_rebin[iE_gamma_rebin]<<" "<<energy_units<<" "
                                                   << gamma_decay_p_emissivity_rebin [iE_gamma_rebin]<<" "
                                                   << gamma_decay_he_emissivity_rebin[iE_gamma_rebin]<<endl;
    }

  emiss_band*=log(E_gamma_rebin[1]/E_gamma_rebin[0]);

  if(debug==1)cout<<"emissivity band: emiss_band="<<emiss_band<<endl;

  return emiss_band;
};

///////////////////////////////////////////////////////////////////

valarray<double>  ProductionMatrices::emissivity_band
          (valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin,
           valarray<double> E_proton_rebin_,
           valarray<double>E_gamma_band_min, valarray<double> E_gamma_band_max, double E_gamma_factor)
{

  cout<<"ProductionMatrices::valarray<double> emissivity_band "<<endl;

  int nE_gamma_band=E_gamma_band_min.size();
  valarray<double> emiss_band;
  emiss_band.resize(nE_gamma_band);
  emiss_band=0;//zero'd AWS20120327
  
  for (int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)
  {
  
   emiss_band[iE_gamma_band] = emissivity_band
                              ( proton_spectrum_rebin,helium_spectrum_rebin,
                                E_proton_rebin_,
                                E_gamma_band_min[iE_gamma_band], E_gamma_band_max[iE_gamma_band],E_gamma_factor);

  
  cout<<"emissivity band valarray<double>:"
             <<" E_gamma_band_min="<< E_gamma_band_min[iE_gamma_band]
             <<" E_gamma_band_max="<< E_gamma_band_max[iE_gamma_band]
             <<" emiss_band="<<emiss_band[iE_gamma_band]<<endl;

  }//  iE_gamma_band

  return emiss_band;
};
//////////////////////////////////////////////////////////////////

 valarray<double> ProductionMatrices::  emissivity_band
                   (valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin)
 {  
 // version using gamma_band matrices precomputed with gen_gamma_band_matrices
 
 cout<<"ProductionMatrices::valarray<double> emissivity_band using precomputed matrices for bands "<<endl;

  int debug=1;

  // matrix precomputed
  int nE_gamma_band= gamma_band_p_matrix [0].size();

  valarray<double> emiss_band;
  emiss_band.resize(nE_gamma_band);
  emiss_band=0;


 for (int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)
  {

  for(int iE_proton_rebin=0; iE_proton_rebin<nE_proton_rebin; iE_proton_rebin++)
   {
     //double g    = E_proton_rebin[iE_proton_rebin]/m_p;// Lorentz factor of proton
     //double beta = sqrt(1.0-1.0/g/g);                  // velocity/c     of proton

     // energy is total energy per nucleon in GeV
     // since energy is per nucleon, protons and helium have same velocity
     // but velocity not needed if spectra are already fluxes. Keep for case of CR densities e.g. for GALPROP

    emiss_band[ iE_gamma_band ]+= 
                                                        proton_spectrum_rebin     [iE_proton_rebin]
                                                      * gamma_band_p_matrix  [iE_proton_rebin][iE_gamma_band]
                                                      * E_proton_rebin[iE_proton_rebin];

    emiss_band[ iE_gamma_band ]+=  
                                                        helium_spectrum_rebin      [iE_proton_rebin]
                                                      * gamma_band_he_matrix [iE_proton_rebin][iE_gamma_band]
                                                      * E_proton_rebin[iE_proton_rebin];

 
    if(debug>=2)
      cout<<"emissivity_band ::using precomputed matrices iE_gamma_band ="<<iE_gamma_band  
        <<" E_proton_rebin= "   << E_proton_rebin[iE_proton_rebin]
        <<" proton spectrum="<<proton_spectrum_rebin[iE_proton_rebin]
        <<" Helium spectrum="<<helium_spectrum_rebin[iE_proton_rebin]
	<<" matrix="<<gamma_band_p_matrix  [iE_proton_rebin][iE_gamma_band]
	<<" emiss_band= "<<emiss_band[ iE_gamma_band ]
	<<" energy_units="<<energy_units
        <<endl;

  

   }//iE_proton_rebin
  }// iE_gamma_band

  emiss_band *=  log(E_proton_rebin[1]/E_proton_rebin[0]); // since log scale integration
  
  emiss_band *= 1.0e-27; // cross-section is in millibarn GeV-1
  
  double H_fraction=0.9;  // Huang etal is per atom of ISM with H fraction 0.9. Convert to per H atom standard
  emiss_band /= H_fraction;
  
    if(debug>=1)
    for (int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)
     cout<<"emissivity_band ::result using precomputed matrices iE_gamma_band ="<<iE_gamma_band  
	<<" emiss_band= "<<emiss_band[ iE_gamma_band ]
	<<" energy_units="<<energy_units
        <<endl;

  return emiss_band;
}
///////////////////////////////////////////////////////////////////

int ProductionMatrices::gen_gamma_band_matrices
               ( valarray<double> E_proton_rebin_,
                 valarray<double>E_gamma_band_min, valarray<double> E_gamma_band_max, double E_gamma_factor)
{

  cout<<"ProductionMatrices::gen_gamma_band_matrices "<<endl;



  int debug=1;

 int nE_gamma_band=E_gamma_band_min.size();

  gamma_band_p_matrix .resize(E_proton_rebin_.size());
  gamma_band_he_matrix.resize(E_proton_rebin_.size());

 //for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++) AWS20120405 nE_proton_rebin not yet defined

 for(int iE_proton_rebin=0;iE_proton_rebin< E_proton_rebin_.size() ;iE_proton_rebin++)
  {
   gamma_band_p_matrix [iE_proton_rebin].resize(nE_gamma_band);
   gamma_band_he_matrix[iE_proton_rebin].resize(nE_gamma_band);
  }

  
  if(debug>=1)
  for(int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)
         cout<<"gen_gamma_band_matrices: input"
             <<" E_gamma_band_min="<< E_gamma_band_min[iE_gamma_band]
             <<" E_gamma_band_max="<< E_gamma_band_max[iE_gamma_band]
             <<endl;

  valarray<double> E_gamma_rebin_work;


 for(int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)
 {

  int nE_gamma_rebin_=
   log(E_gamma_band_max[iE_gamma_band]/E_gamma_band_min[iE_gamma_band])/log(E_gamma_factor)+1.001;//redone in rebin ? check

  if(debug>=1) cout<<"ProductionMatrices::gen_gamma_band_matrices: nE_gamma_rebin_="<<nE_gamma_rebin_<<endl;

  E_gamma_rebin_work.resize(nE_gamma_rebin_);

  if(debug>=1) cout<<"ProductionMatrices::gen_gamma_band_matrices: after E_gamma_rebin_work.resize"<<endl;

  for(int iE_gamma_rebin=0;iE_gamma_rebin<nE_gamma_rebin_;iE_gamma_rebin++)
        E_gamma_rebin_work[iE_gamma_rebin] = E_gamma_band_min[iE_gamma_band] * pow(E_gamma_factor,iE_gamma_rebin);

  

  rebin(E_proton_rebin_,E_gamma_rebin_work); // NB resets E_proton_rebin, E_gamma_rebin in class.

  
  for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
  {
   double emiss_band_p=0;
   double emiss_band_he=0;
  
   for(int iE_gamma_rebin=0;iE_gamma_rebin<nE_gamma_rebin;iE_gamma_rebin++)
   {

      emiss_band_p += gamma_decay_p_matrix_rebin [iE_proton_rebin] [iE_gamma_rebin]*E_gamma_rebin[iE_gamma_rebin];
      emiss_band_he+= gamma_decay_he_matrix_rebin[iE_proton_rebin] [iE_gamma_rebin]*E_gamma_rebin[iE_gamma_rebin];
      

      if(debug>=3)cout<<"gen_gamma_band_matrices: "<<" E_proton_rebin="<<E_proton_rebin [iE_proton_rebin]
                                                   <<" E_gamma= "      <<E_gamma_rebin[iE_gamma_rebin]
                                                   <<" "<<energy_units
	                      	                   <<" emiss_band_p=" << emiss_band_p
		                                   <<" emiss_band_he="<< emiss_band_he
                                                   <<endl;
    }

  emiss_band_p *=log(E_gamma_rebin[1]/E_gamma_rebin[0]);
  emiss_band_he*=log(E_gamma_rebin[1]/E_gamma_rebin[0]);

 


	 gamma_band_p_matrix  [iE_proton_rebin][iE_gamma_band] = emiss_band_p;
	 gamma_band_he_matrix [iE_proton_rebin][iE_gamma_band] = emiss_band_he;

       if(debug>=2)
         cout<<"gen_gamma_band_matrices: result"
             <<" E_proton_rebin="  << E_proton_rebin  [iE_proton_rebin]
             <<" E_gamma_band_min="<< E_gamma_band_min[iE_gamma_band]
             <<" E_gamma_band_max="<< E_gamma_band_max[iE_gamma_band]
             <<" "<<energy_units
             <<" emiss_band_p="<<emiss_band_p
             <<" emiss_band_he="<<emiss_band_he
             <<endl;


    }// iE_gamma_band

  }// iE_proton_rebin





 if(debug>=1)
 for(int iE_proton_rebin=0;iE_proton_rebin<nE_proton_rebin;iE_proton_rebin++)
 for(int iE_gamma_band=0; iE_gamma_band< nE_gamma_band; iE_gamma_band++)

         cout<<"gen_gamma_band_matrices: result matrices"
             <<" E_proton_rebin="  << E_proton_rebin  [iE_proton_rebin]
             <<" E_gamma_band_min="<< E_gamma_band_min[iE_gamma_band]
             <<" E_gamma_band_max="<< E_gamma_band_max[iE_gamma_band]
             <<" "<<energy_units
             <<" p: "<< gamma_band_p_matrix  [iE_proton_rebin][iE_gamma_band]
             <<" He:"<< gamma_band_he_matrix [iE_proton_rebin][iE_gamma_band]
             <<endl;



  return 0;
};
//////////////////////////////////////////////////////////////////

int ProductionMatrices::test()
{
  cout<<"ProductionMatrices::test"<<endl;

  string energy_type;
  string energy_units_="MeV";

  energy_type="kinetic_energy";

  init(energy_type,energy_units_);

  energy_type="total_energy";
  init(energy_type,energy_units_);



  /* invalid type: tests check and exit
  energy_type="junk";
  init(energy_type,energy_units_);
  */

  read();

  double E_proton_min,E_proton_max, E_proton_factor;
  double E_gamma_min,E_gamma_max, E_gamma_factor;

  E_proton_min   =  1.0;  //  in GeV/nucleon NB total not Ek
  E_proton_min   =  0.4;  //  for case of kinetic energy, but should work also for total
  E_proton_max   =  1.0e3;// = 1   TeV/nucleon
  E_proton_factor=  1.10;

  E_gamma_min    =  1.0e-1;//  in GeV
  E_gamma_max    =  1.0e3; // = 1 TeV
  E_gamma_factor =  1.10;



  int nE_proton_rebin_;
  int nE_gamma_rebin_ ;      

  nE_proton_rebin_ = log(E_proton_max/E_proton_min)/log(E_proton_factor) + 1.001;
  nE_gamma_rebin_  = log(E_gamma_max /E_gamma_min) /log(E_gamma_factor ) + 1.001;

  valarray<double> E_proton_rebin_;
  valarray<double> E_gamma_rebin_ ;

  E_proton_rebin_.resize(nE_proton_rebin_);
  E_gamma_rebin_ .resize( nE_gamma_rebin_);

  for(int i=0;i<nE_proton_rebin_;i++)E_proton_rebin_[i]=  E_proton_min *  pow(E_proton_factor,i);
  for(int i=0;i<nE_gamma_rebin_ ;i++)E_gamma_rebin_ [i]=  E_gamma_min  *  pow(E_gamma_factor ,i);

  rebin( E_proton_rebin_,  E_gamma_rebin_);




  // rebin( E_proton_rebin_,  E_gamma_rebin_,"test units" ); causes exit as required


  valarray<double> proton_spectrum, helium_spectrum;
  proton_spectrum.resize( nE_proton_rebin_);
  helium_spectrum.resize( nE_proton_rebin_);

  int             nE_gamma_band;
  valarray<double> E_gamma_band_min_;
  valarray<double> E_gamma_band_max_;



 

  // Shikaze et al 2007 http://adsabs.harvard.edu/abs/2007APh....28..154S
  // Fig 8 and 9 parameters
  // flux = A beta^P1 R^-P2

  cout<<"emissivity from Shikaze etal protons + Helium"<<endl;

  // protons
  double A=1.94e4*1e-4;// from m-2 sr-1 s-1 GeV-1 to cm-2  sr-1 s-1 GeV-1
  double P1=0.70;
  double P2=2.76;
  double m_p=0.938;// proton mass in GeV


 int test_without_CR_Spectrum=0;
 if( test_without_CR_Spectrum==1)
 {
  proton_spectrum=0.;
  helium_spectrum=0.;

  for(int i=0;i<nE_proton_rebin_;i++)
    {
      double Ek   = E_proton_rebin_[i]-m_p;// kinetic energy per nucleon
      double g    = E_proton_rebin_[i]/m_p;// Lorentz factor of proton
      double beta = sqrt(1.0-1.0/g/g);                  // velocity/c     of proton
      double R    = sqrt(pow(E_proton_rebin_[i],2)-pow(m_p,2)); // rigidity = momentum

      proton_spectrum[i]=A*pow(beta,P1)*pow(R,-P2);

      

     cout<<"E total= "<< E_proton_rebin_[i]<< " GeV Ek="<<Ek<<" GeV beta="<<beta<<" R="<<R 
                      <<" Shikaze proton spectrum="<< proton_spectrum[i]<<"  cm-2  sr-1 s-1 GeV-1"<<endl;

    }


  // Helium

         A=7.10e3*1e-4;// from m-2  sr-1 s-1 (GeV/n)-1 to cm-2  sr-1 s-1 (GeV/n)-1
         P1=0.50;
         P2=2.78;

  for(int i=0;i<nE_proton_rebin_;i++)
    {
      double Ek, g,beta,R;

      
	{

	     double Z=2.0;
             Ek   = E_proton_rebin_[i] - m_p;                    // kinetic energy per nucleon
	     g    = E_proton_rebin_[i]/m_p;                   // Lorentz factor of helium
             beta = sqrt(1.0-1.0/g/g);                               // velocity/c     of helium
             R    = sqrt(pow(4.*E_proton_rebin_[i],2)-pow(4.*m_p,2))/Z; // rigidity = particle momentum/Z  AWS20120319

             helium_spectrum[i]=A*pow(beta,P1)  *pow(R,-P2) ; 
	}
      cout<<"E total= "<< E_proton_rebin_[i]<< " GeV Ek/n="<<Ek  <<" GeV beta="<<beta<<" R="<<R 
                       <<" Shikaze helium spectrum="<< helium_spectrum[i]<<"  cm-2  sr-1 s-1 (GeV/n)-1"<<endl;
    }

  

 

  
  
  //  for(int i=0;i<nE_proton_rebin_;i++)cout<<"actually used for emissivity: E total= "<< E_proton_rebin_[i]
  //                                           <<" protons="<<proton_spectrum[i]<<" helium="<<helium_spectrum[i]<<endl;


    emissivity(proton_spectrum,helium_spectrum);

    cout<<endl<<"after first rebin and emissivity"<<endl;
    print();

  }// test_without_CR_Spectrum



    // define energy bands

   // for cf values in Fermi 2nd quadrant paper http://adsabs.harvard.edu/abs/2010ApJ...710..133A  Table 1 qHI,1=local
   // qHI=0.2-0.4 GeV: 0.584,  0.4-0.6 GeV:0.224, 0.6-1.0 GeV: 0.168,  1-2 GeV : 0.110, 2-10 GeV :0.048e-26 sr sr-1

  nE_gamma_band=35;
  E_gamma_band_min_.resize(nE_gamma_band);
  E_gamma_band_max_.resize(nE_gamma_band);

  
  E_gamma_band_min_[ 0]= 0.050; E_gamma_band_max_[ 0]= 0.100;//GeV
  E_gamma_band_min_[ 1]= 0.100; E_gamma_band_max_[ 1]= 0.200;

  // 2nd quadrant paper bands
  E_gamma_band_min_[ 2]= 0.200; E_gamma_band_max_[ 2]= 0.400;
  E_gamma_band_min_[ 3]= 0.400; E_gamma_band_max_[ 3]= 0.600;
  E_gamma_band_min_[ 4]= 0.600; E_gamma_band_max_[ 4]= 1.000;
  E_gamma_band_min_[ 5]= 1.000; E_gamma_band_max_[ 5]= 2.000;
  E_gamma_band_min_[ 6]= 2.000; E_gamma_band_max_[ 6]=10.000;

  // another band
  E_gamma_band_min_[ 7]=10.000; E_gamma_band_max_[ 7]=20.000;

  // Abdo etal 2009 HI emissivity paper bands
  E_gamma_band_min_[ 8]= 0.100; E_gamma_band_max_[ 8]= 0.140;//GeV
  E_gamma_band_min_[ 9]= 0.140; E_gamma_band_max_[ 9]= 0.200;
  E_gamma_band_min_[10]= 0.200; E_gamma_band_max_[10]= 0.280;
  E_gamma_band_min_[11]= 0.280; E_gamma_band_max_[11]= 0.400;
  E_gamma_band_min_[12]= 0.400; E_gamma_band_max_[12]= 0.560;
  E_gamma_band_min_[13]= 0.560; E_gamma_band_max_[13]= 0.800;
  E_gamma_band_min_[14]= 0.800; E_gamma_band_max_[14]= 1.130;
  E_gamma_band_min_[15]= 1.130; E_gamma_band_max_[15]= 1.600;
  E_gamma_band_min_[16]= 1.600; E_gamma_band_max_[16]= 2.260;
  E_gamma_band_min_[17]= 2.260; E_gamma_band_max_[17]= 3.200;
  E_gamma_band_min_[18]= 3.200; E_gamma_band_max_[18]= 4.530;
  E_gamma_band_min_[19]= 4.530; E_gamma_band_max_[19]= 6.400;
  E_gamma_band_min_[20]= 6.400; E_gamma_band_max_[20]= 9.050;

  // Ackermann etal 2011 3rd quadrant paper
  E_gamma_band_min_[21]= 0.100; E_gamma_band_max_[21]= 0.140;//GeV
  E_gamma_band_min_[22]= 0.140; E_gamma_band_max_[22]= 0.200;
  E_gamma_band_min_[23]= 0.200; E_gamma_band_max_[23]= 0.280;
  E_gamma_band_min_[24]= 0.280; E_gamma_band_max_[24]= 0.400;
  E_gamma_band_min_[25]= 0.400; E_gamma_band_max_[25]= 0.560;
  E_gamma_band_min_[26]= 0.560; E_gamma_band_max_[26]= 0.800;
  E_gamma_band_min_[27]= 0.800; E_gamma_band_max_[27]= 1.130;
  E_gamma_band_min_[28]= 1.130; E_gamma_band_max_[28]= 1.600;
  E_gamma_band_min_[29]= 1.600; E_gamma_band_max_[29]= 2.260;
  E_gamma_band_min_[30]= 2.260; E_gamma_band_max_[30]= 3.200;
  E_gamma_band_min_[31]= 3.200; E_gamma_band_max_[31]= 4.530;
  E_gamma_band_min_[32]= 4.530; E_gamma_band_max_[32]= 6.400;
  E_gamma_band_min_[33]= 6.400; E_gamma_band_max_[33]= 9.050;
  E_gamma_band_min_[34]= 9.050; E_gamma_band_max_[34]=25.600;

  valarray<double> emiss_measured,emiss_measured_from_table;
  emiss_measured           .resize(nE_gamma_band);
  emiss_measured = 0.0;
  emiss_measured[ 2]=0.584e-26;
  emiss_measured[ 3]=0.224e-26;
  emiss_measured[ 4]=0.168e-26;
  emiss_measured[ 5]=0.110e-26;
  emiss_measured[ 6]=0.048e-26;
  for(int i=2;i<= 6;i++) 
   cout<<"Abdo etal 2010 ApJ 710, 133 2nd quadrant Goulds Belt measured emissivity: E_gamma="<<E_gamma_band_min_[i]
        <<" - "<<E_gamma_band_max_[i]
        <<" GeV =" << emiss_measured[i]<<endl;
  


  // Table 1 of Abdo etal 703, 1249, 2009 (HI emissivity paper)
  emiss_measured[ 8] = 1.04e-24;
  emiss_measured[ 9] = 1.67e-24;
  emiss_measured[10] = 1.91e-24;
  emiss_measured[11] = 2.11e-24;
  emiss_measured[12] = 2.33e-24;
  emiss_measured[13] = 2.20e-24;
  emiss_measured[14] = 2.17e-24;
  emiss_measured[15] = 1.88e-24;
  emiss_measured[16] = 1.72e-24;
  emiss_measured[17] = 1.15e-24;
  emiss_measured[18] = 1.10e-24;
  emiss_measured[19] = 1.12e-24;
  emiss_measured[20] = 0.71e-24;
  

  // convert from MeV^2 s-1 sr-1 MeV-1 to  s-1 sr-1 using geometric mean for E^2
  for(int i=8;i<=20;i++)
  {
   emiss_measured[i]*=1.0e-3; // MeV-> GeV
   emiss_measured[i]*=(E_gamma_band_max_[i]- E_gamma_band_min_[i]);
   emiss_measured[i]/=(E_gamma_band_max_[i]* E_gamma_band_min_[i]);
   cout<<"Abdo etal 2009 703, 1249, 2009 (HI emissivity paper) measured emissivity: E_gamma="
        <<E_gamma_band_min_[i]<<" - "<<E_gamma_band_max_[i]
        <<" GeV =" << emiss_measured[i]<<endl;
  }

  // Table 2 of Ackermann etal ApJ 726, 81, 2011   (3rd quad paper)
  // Ts=250K, qHI,1 = local
  emiss_measured[21] = 1.35e-24;
  emiss_measured[22] = 1.56e-24;
  emiss_measured[23] = 1.82e-24;
  emiss_measured[24] = 2.00e-24;
  emiss_measured[25] = 1.95e-24;
  emiss_measured[26] = 2.10e-24;
  emiss_measured[27] = 1.90e-24;
  emiss_measured[28] = 1.79e-24;
  emiss_measured[29] = 1.74e-24;
  emiss_measured[30] = 1.37e-24;
  emiss_measured[31] = 0.84e-24;
  emiss_measured[32] = 0.65e-24;
  emiss_measured[33] = 0.54e-24;
  emiss_measured[34] = 0.38e-24;

  // convert from MeV^2 s-1 sr-1 MeV-1 to  s-1 sr-1 using geometric mean for E^2
  for(int i=21;i<=34;i++)
  {
   emiss_measured[i]*=1.0e-3; // MeV-> GeV
   emiss_measured[i]*=(E_gamma_band_max_[i]- E_gamma_band_min_[i]);
   emiss_measured[i]/=(E_gamma_band_max_[i]* E_gamma_band_min_[i]);
   cout<<"Ackermann etal ApJ 726, 81, 2011   (3rd quad paper) measured emissivity: E_gamma="
        <<E_gamma_band_min_[i]<<" - "<<E_gamma_band_max_[i]
        <<" GeV =" << emiss_measured[i]<<endl;
  }

  // finished defining energies and bands


  double E_gamma_band_min;
  double E_gamma_band_max;
  double E_gamma_band_factor=1.01;// for integration over gamma-ray band

  double emiss_band, emiss_band_protons, emiss_band_helium;

  valarray<double>zero_spectrum;
  zero_spectrum.resize(proton_spectrum.size());
  zero_spectrum=0.;


  int  test_proton_helium_separately=0;

  if ( test_proton_helium_separately==1)
  {



  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
    {
     E_gamma_band_min=E_gamma_band_min_[iE_gamma_band];
     E_gamma_band_max=E_gamma_band_max_[iE_gamma_band];

         emiss_band=
    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);

        emiss_band_protons=
    emissivity_band( proton_spectrum,  zero_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);

        emiss_band_helium=
    emissivity_band( zero_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);


	 cout<<"test: Shikaze LIS "                 
             <<" E_gamma_band_min="<< E_gamma_band_min<<" E_gamma_band_max="<< E_gamma_band_max
             <<" emiss_band="<<emiss_band      <<" emiss_band_protons="<<emiss_band_protons    
             <<" emiss_band_helium="<<emiss_band_helium         
             <<" predicted/measured = " <<emiss_band/emiss_measured[iE_gamma_band]
             <<endl;

	 cout<<"Shikaze LIS "                 
             <<" E_gamma="<< E_gamma_band_min<<" -"<< E_gamma_band_max
             <<" GeV predicted emiss="  <<emiss_band   
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band/emiss_measured[iE_gamma_band]
             <<endl;

}


  }//test_proton_helium_separately



  // test multiple bands version

  valarray<double> emiss_band_;
  emiss_band_.resize(nE_gamma_band);

  int  test_multiple_bands=0;

  if(test_multiple_bands==1)
  {

  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" output from multiple bands version: Shikaze LIS "                 
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" GeV predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

  }//test_multiple_bands==1

 ////////////////////////////////////////////////


  int test_varying_modulation=0;

 if (test_varying_modulation==1)
 {

 cout<<"emissivity from varying modulation of  protons + Helium"<<endl;

 /*
Gleeson & Axford,  ApJ 154, 1011 (1968)
The formulae here are based on equation (16) of that paper.

 J(E)           J(E + |Z| e phi)
________   =    _________________________ 
E^2-Eo^2         (E + |Z| e phi)^2 - Eo^2        

where E = total energy of particle, Eo = rest mass energy of particle.

see also galplot: modulate.cc
 */

 // Shikaze LIS uses modulation 591 MV for 1998 data, so compute relative to this.

 for(double Phi=0.;Phi< 600.;Phi+= 50.)
 {

   //          Phi= 91. ;// modulation parameter in standard MV units
  double PhiGV=Phi/1000.;// modulation parameter in GV 

   A=1.94e4*1e-4;// from m-2 sr-1 s-1 GeV-1 to cm-2  sr-1 s-1 GeV-1
   P1=0.70;
   P2=2.76;
   m_p=0.938;// proton mass in GeV

   proton_spectrum=0.;
   helium_spectrum=0.;

  for(int i=0;i<nE_proton_rebin_;i++)
    {
      double Ek   = E_proton_rebin_[i]-m_p;// kinetic energy per nucleon
      double g    = E_proton_rebin_[i]/m_p;// Lorentz factor of proton
      double beta = sqrt(1.0-1.0/g/g);                  // velocity/c     of proton
      double R    = sqrt(pow(E_proton_rebin_[i],2)-pow(m_p,2)); // rigidity = momentum

      double        E_prime = E_proton_rebin_[i]+PhiGV;// total energy prime
      double        g_prime = E_prime/m_p;
      double     beta_prime = sqrt(1.0-1.0/g_prime/g_prime);   
      double        R_prime = sqrt(pow(E_prime,2)-pow(m_p,2)); // rigidity = momentum
      double        factor  = ( pow(  E_proton_rebin_[i],2)-pow(m_p,2)) / ( pow(  E_prime,2)-pow(m_p,2))   ;

      proton_spectrum[i]=A*pow(beta_prime,P1)*pow(R_prime,-P2)* factor;

      cout<<"E total = "<< E_proton_rebin_[i] <<" Ek="<<Ek<< " E_prime="<<E_prime<<" R_prime="<<R_prime
                        <<" Shikaze modulated proton spectrum="<< proton_spectrum[i]<<"  cm-2  sr-1 s-1 GeV-1" <<endl;
    }

  // Helium

         A=7.10e3*1e-4;// from m-2  sr-1 s-1 GeV-1 to cm-2  sr-1 s-1 GeV-1
         P1=0.50;
         P2=2.78;

  for(int i=0;i<nE_proton_rebin_;i++)
    {
     
     
      {
      double Z=2.;
      double Ek   = E_proton_rebin_[i] - m_p;// kinetic energy per nucleon
      double g    = E_proton_rebin_[i] / m_p;// Lorentz factor of particle
      double beta = sqrt(1.0-1.0/g/g);                  // velocity/c     of particle
      double R    = sqrt(pow(4.*E_proton_rebin_[i],2)-pow(4.*m_p,2))/Z;// rigidity = particle momentum/Z 
    


      double        E_prime = E_proton_rebin_[i] +Z *PhiGV/4.0;        // total energy per nucleon prime
      double        g_prime = E_prime/m_p;                             // Lorentz factor of particle
      double     beta_prime = sqrt(1.0-1.0/g_prime/g_prime);           // velocity/c     of particle  
      double        R_prime = sqrt(pow(4.*E_prime,2)-pow(4.*m_p,2))/Z; // rigidity = particle momentum/Z
      double        factor  = ( pow(  4.*E_proton_rebin_[i],2)-pow(4.*m_p,2)) / ( pow(  4.*E_prime,2)-pow(4.*m_p,2))   ;

      helium_spectrum[i]=A*pow(beta_prime,P1)*pow(R_prime,-P2)* factor;

      cout<<"E total = "<< E_proton_rebin_[i] <<" Ek="<<Ek<<" GeV/n  E_prime="<<E_prime<<" R_prime="<<R_prime
                        <<" Shikaze modulated helium spectrum="<< helium_spectrum[i]<<"  cm-2  sr-1 s-1 GeV-1" <<endl;
      }


     
    }

  
  
  

         

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
    {
     E_gamma_band_min=E_gamma_band_min_[iE_gamma_band];
     E_gamma_band_max=E_gamma_band_max_[iE_gamma_band];

         emiss_band=
    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);

        emiss_band_protons=
    emissivity_band( proton_spectrum,  zero_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);

        emiss_band_helium=
    emissivity_band( zero_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min, E_gamma_band_max, E_gamma_band_factor);


	cout<<"test: Shikaze modulated Phi="<<Phi<<" delta Phi="<<591-Phi
             <<" E_gamma_band_min="<< E_gamma_band_min<<" E_gamma_band_max="<< E_gamma_band_max
             <<" emiss_band="<<emiss_band      <<" emiss_band_protons="<<emiss_band_protons    
             <<" emiss_band_helium="<<emiss_band_helium                        <<endl;



	 cout<<"Shikaze modulated Phi="<<Phi<<" delta Phi="<<591-Phi
             <<" E_gamma="<< E_gamma_band_min<<" -"<< E_gamma_band_max
             <<" GeV predicted emiss="  <<emiss_band   
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band/emiss_measured[iE_gamma_band]
             <<endl;
}

  // test multiple band version

  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<"result from multiple bands version: Shikaze modulated Phi="<<Phi<<" delta Phi="<<591-Phi         
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" GeV predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;
  

 }// for Phi


 }//test_varying_modulation==1

  ////////////////////////////////////////////////






 // test CR_Spectrum

  CR_Spectrum cr_spectrum;
  cr_spectrum.test();

  cout<< "--- ProductionMatrices:: test : using CR_Spectrum"<<endl;

  

  string name;
 
  valarray<double>parameters;
  parameters.resize(10);

  //////////////////////////////////////////////////////

  int test_using_CR_spectrum=0;

  if(test_using_CR_spectrum==1)
  {

 for(double Phi=300.;Phi< 620.;Phi+= 20.)
 {

  parameters[0]=Phi ;// modulation in MV

  int debug=1;

  name="Shikaze protons";
  cr_spectrum.init(name,parameters, debug);
  proton_spectrum=cr_spectrum.flux(E_proton_rebin);

  name="Shikaze Helium";
  cr_spectrum.init(name,parameters, debug);
  helium_spectrum=cr_spectrum.flux(E_proton_rebin);

  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" result from multiple bands CR_Spectrum  Shikaze modulated"   
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" GeV predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

 

  }// Phi


  }//test_using_CR_spectrum==1



  /////////////////////////////////////

// test MeV version

  cout<<"testing MeV version"<<endl;

  energy_units_="MeV";
  valarray<double>  E_proton_rebin_MeV;
  E_proton_rebin_MeV.resize(E_proton_rebin.size());
  E_proton_rebin_MeV      = E_proton_rebin*1.e3; //GeV->MeV

  valarray<double>  E_gamma_rebin_MeV;
  E_gamma_rebin_MeV.resize(E_gamma_rebin.size());
  E_gamma_rebin_MeV      = E_gamma_rebin*1.e3; //GeV->MeV


  valarray<double> E_gamma_band_min_MeV,E_gamma_band_max_MeV;
  E_gamma_band_min_MeV.resize(nE_gamma_band);
  E_gamma_band_max_MeV.resize(nE_gamma_band);
  E_gamma_band_min_MeV =E_gamma_band_min_*1e3; //GeV->MeV
  E_gamma_band_max_MeV =E_gamma_band_max_*1e3; //GeV->MeV


  double Phi=591;// test case of no modulation difference first
  parameters[0]=Phi ;// modulation in MV



  name="Shikaze protons";
  int debug=0;
  cr_spectrum.init(name,parameters, debug);
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_); // still GeV but can use like this. use E_proton_rebin_ not E_proton_rebin since latter is in class
  
  name="Shikaze Helium";
  cr_spectrum.init(name,parameters, debug);
  helium_spectrum=cr_spectrum.flux(E_proton_rebin_); // still GeV but can use like this

  // GeV units
  energy_units_="GeV";

  /* test rebin and emissivity
  rebin( E_proton_rebin , E_gamma_rebin);

  emissivity(proton_spectrum, helium_spectrum);
  cout<<"checking matrices after rebin and emissivity for GeV"<<endl;
  print();
  */


 for(int i=0;i<nE_proton_rebin_;i++)  cout<<"actually used for checking  emissivity: E_proton_rebin "
				 	  << E_proton_rebin[i]<<" E_proton_rebin_=  "<< E_proton_rebin_[i]
                                          <<" E_proton_rebin_MeV=  "<< E_proton_rebin_MeV[i]
                                          <<" protons="<<proton_spectrum[i]<<" helium="<<helium_spectrum[i]
                                          <<" energy_units="<<energy_units_  <<endl;

  emiss_band_.resize(nE_gamma_band);
  emiss_band_=0.;
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                                  E_proton_rebin,
				  E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" checking result for "  
	     <<" energy_units="<<energy_units 
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;
 


  // MeV units
  energy_units_="MeV";
  proton_spectrum*=1.e-3;//GeV^-1 -> MeV^-1
  helium_spectrum*=1.e-3;//GeV^-1 -> MeV^-1

  /* test rebin and emissivity
  rebin( E_proton_rebin_MeV , E_gamma_rebin_MeV);

  emissivity(proton_spectrum, helium_spectrum);
  cout<<"checking after matrices after rebin and emissivity for MeV"<<endl;

  print();
  */


  for(int i=0;i<nE_proton_rebin_;i++)cout<<"actually used for checking  emissivity: E_proton_rebin "
					 << E_proton_rebin[i]<<" E_proton_rebin_=  "<< E_proton_rebin_[i]
                                          <<" E_proton_rebin_MeV=  "<< E_proton_rebin_MeV[i]
                                          <<" protons="<<proton_spectrum[i]<<" helium="<<helium_spectrum[i]
                                          <<" energy_units="<<energy_units_  <<endl;
  emiss_band_.resize(nE_gamma_band);
  emiss_band_=0.;
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                                  E_proton_rebin_MeV,
				   E_gamma_band_min_MeV, E_gamma_band_max_MeV, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" checking result for "  
	     <<" energy_units="<<energy_units 
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_MeV[iE_gamma_band]<<" -"<< E_gamma_band_max_MeV[iE_gamma_band]
             <<" predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

  // the above comparison showed correct emissivities also in bands.

  // for(double Phi=300.;Phi< 620.;Phi+= 20.)
   Phi=591;// test case of no modulation difference first
 {

  parameters[0]=Phi ;// modulation in MV

  debug=1;

  name="Shikaze protons";
  cr_spectrum.init(name,parameters, debug);
  //  proton_spectrum=cr_spectrum.flux(E_proton_rebin); // still GeV but can use after conversion
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_); // NB previous invocation of emissivity_band has modified E_proton_bin in class
  proton_spectrum*=1.e-3;//GeV^-1 -> MeV^-1

  name="Shikaze Helium";
  cr_spectrum.init(name,parameters, debug);
  //  helium_spectrum=cr_spectrum.flux(E_proton_rebin); // still GeV but can use after conversion
  helium_spectrum=cr_spectrum.flux(E_proton_rebin_); // NB previous invocation of emissivity_band has modified E_proton_bin in class
  helium_spectrum*=1.e-3;//GeV^-1 -> MeV^-1

  energy_units_="MeV";

  

  for(int i=0;i<nE_proton_rebin_;i++)cout<<"actually used for checking again emissivity: E_proton_rebin "
					 << E_proton_rebin[i]<<" E_proton_rebin_=  "<< E_proton_rebin_[i]
                                          <<" E_proton_rebin_MeV=  "<< E_proton_rebin_MeV[i]
                                          <<" protons="<<proton_spectrum[i]<<" helium="<<helium_spectrum[i]
                                          <<" energy_units="<<energy_units_  <<endl;


  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                                  E_proton_rebin_MeV,
				   E_gamma_band_min_MeV, E_gamma_band_max_MeV, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" checking again result from multiple for "  
	     <<" energy_units="<<energy_units_ 
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_MeV[iE_gamma_band]<<" -"<< E_gamma_band_max_MeV[iE_gamma_band]
             <<" predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

 

  }// Phi


 // Shikaze CR_Spectrum MeV version

  Phi=591;// test case of no modulation difference first

 for(double Phi=300.;Phi< 620.;Phi+= 20.)
 {

  parameters[0]=Phi ;// modulation in MV

  debug=1;

  name="Shikaze protons MeV";
  cr_spectrum.init(name,parameters, debug);
  
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 
  

  name="Shikaze Helium MeV";
  cr_spectrum.init(name,parameters, debug);

  helium_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 
  

  energy_units_="MeV";

  
  for(int i=0;i<nE_proton_rebin_;i++)cout<<"actually used for checking Shikaze MeV CR_Spectrum emissivity: E_proton_rebin= "
					 << E_proton_rebin[i]<<" E_proton_rebin_=  "<< E_proton_rebin_[i]
                                          <<" E_proton_rebin_MeV=  "<< E_proton_rebin_MeV[i]
					  <<" Phi="<<Phi
                                          <<" protons="<<proton_spectrum[i]<<" helium="<<helium_spectrum[i]
                                          <<" energy_units="<<energy_units_  <<endl;


  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                                  E_proton_rebin_MeV,
				   E_gamma_band_min_MeV, E_gamma_band_max_MeV, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" checking Shikaze MeV CR_Spectrum for "  
	     <<" energy_units="<<energy_units_ 
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_MeV[iE_gamma_band]<<" -"<< E_gamma_band_max_MeV[iE_gamma_band]
             <<" predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

 

  }// Phi

 ///////////////////////////////////////////

int test_gamma_bands_matrices =1;
 if (test_gamma_bands_matrices==1)
 {
   cout<<"testing emissivity for  gamma-ray energy bands using precomputed matrices"<<endl;

   /* to avoid setting directly
  energy_units="GeV";

  gen_gamma_band_matrices(E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_,  E_gamma_band_factor);
   */

   // this should not work since init(energy_type,energy_units) has not been called: test this 
  
  energy_units_="GeV";

  
  gen_gamma_band_matrices(E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_,  E_gamma_band_factor);
  
   

  Phi=591;
  parameters[0]=Phi ;// modulation in MV



  name="Shikaze protons";
  int debug=0;
  cr_spectrum.init(name,parameters, debug);
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_); // GeV
  
  name="Shikaze Helium";
  cr_spectrum.init(name,parameters, debug);
  helium_spectrum=cr_spectrum.flux(E_proton_rebin_); 

  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum);


  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" using precomputed matrices "  
	     <<" energy_units="<<energy_units 
	     <<" energy_units_="<<energy_units_ 
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" predicted emiss="      <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

  /////////////////////


  energy_units="MeV";

  gen_gamma_band_matrices(E_proton_rebin_MeV,
                          E_gamma_band_min_MeV, E_gamma_band_max_MeV,  E_gamma_band_factor);

  name="Shikaze protons MeV";
  cr_spectrum.init(name,parameters, debug);
  
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 
  

  name="Shikaze Helium MeV";
  cr_spectrum.init(name,parameters, debug);

  helium_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 

  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum);

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" using precomputed matrices "  
	     <<" energy_units="<<energy_units 
	     <<" energy_type ="<<energy_type
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_MeV[iE_gamma_band]<<" -"<< E_gamma_band_max_MeV[iE_gamma_band]
             <<" predicted emiss="      <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;


  ///////////////

  energy_units="MeV";

  energy_type="kinetic_energy";
  init(energy_type,energy_units);

  

  gen_gamma_band_matrices(E_proton_rebin_MeV,
                          E_gamma_band_min_MeV, E_gamma_band_max_MeV,  E_gamma_band_factor);

  name="Shikaze protons kinetic energy MeV";
  cr_spectrum.init(name,parameters, debug);
  
  proton_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 
  

  name="Shikaze Helium kinetic energy MeV";
  cr_spectrum.init(name,parameters, debug);

  helium_spectrum=cr_spectrum.flux(E_proton_rebin_MeV); 

  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum);

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<" using precomputed matrices "  
	     <<" energy_units="<<energy_units 
	     <<" energy_type ="<<energy_type  
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_MeV[iE_gamma_band]<<" -"<< E_gamma_band_max_MeV[iE_gamma_band]
             <<" predicted emiss="      <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;


 }//test_gamma_bands_matrices


 ///////////////////////////////////////////////////////
   int test_Gaisser_protons=0;

 if (test_Gaisser_protons==1)
 {

 for(double Phi=0;Phi<=500.;Phi+= 50.)
 {

  parameters[0]=Phi ;// modulation in MV

  int debug=1;

  name="Gaisser protons";
  cr_spectrum.init(name,parameters, debug);
  proton_spectrum=cr_spectrum.flux(E_proton_rebin);

  name="Gaisser Helium high";
  cr_spectrum.init(name,parameters, debug);
  helium_spectrum=cr_spectrum.flux(E_proton_rebin);


  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<"CR_Spectrum  Gaisser demodulated, p+He high"   
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" GeV predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;

  name="Gaisser Helium low";
  cr_spectrum.init(name,parameters, debug);
  helium_spectrum=cr_spectrum.flux(E_proton_rebin);


  emiss_band_.resize(nE_gamma_band);
  emiss_band_=    emissivity_band( proton_spectrum,  helium_spectrum,
                          E_proton_rebin_,
                          E_gamma_band_min_, E_gamma_band_max_, E_gamma_band_factor);
  

  for(int iE_gamma_band=0;  iE_gamma_band<nE_gamma_band; iE_gamma_band++)
  	     cout<<"CR_Spectrum  Gaisser demodulated, p+He low "   
	     <<" Phi="<<Phi<<" MV"
             <<" E_gamma="<< E_gamma_band_min_[iE_gamma_band]<<" -"<< E_gamma_band_max_[iE_gamma_band]
             <<" GeV predicted emiss="  <<emiss_band_[iE_gamma_band] 
             <<" measured = "           <<emiss_measured[iE_gamma_band]
             <<" predicted/measured = " <<emiss_band_[iE_gamma_band]/emiss_measured[iE_gamma_band]
             <<endl;



  }// Phi


 }// test_Gaisser_protons==1



 

  return 0;
};
