// sample program to compute gamma-ray emissivities using Huang et al. Astroparticle Physics, 27, 429 2007  matrices
// for given cosmic-ray proton and Helium spectra. Target is interstellar medium as defined by Huang etal,
// emissivity is per hydrogen atom (not per atom as in Huang etal, corrected for 10% Helium). 
// The class used is in  ProductionMatrices.cc
// For more details see the documentation there.
// This sample program can be edited to provide the required functionality.

// Requires the following files:
// source:
// CR_Spectrum.cc  CR_Spectrum.h  ProductionMatrices.cc  ProductionMatrices.h  test_ProductionMatrices.cc
// data:
// espectra_gamma.decay.he.matrix.data  espectra_gamma.decay.p.matrix.data  prodxsection.he.matrix.data  prodxsection.p.matrix.data

//  to compile and run this program:
//  g++ *cc -O3 (also tested with intel icpc)
//  ./a.out  (lots of screen output)
// to see just the sample output:
// ./a.out | grep sample

// Version 1.0  13 Apr 2012
// Andy Strong, MPE, Garching, aws@mpe.mpg.de
// Available from http://www.mpe.mpg.de/~aws/propagate.html

using namespace std;
#include<iostream>
#include<fstream>
#include<cstring>
#include<cstdlib>
#include<vector>
#include<valarray>

#include"ProductionMatrices.h"


/////////////////////////////
int main()
{
 cout<<" Start of test_ProductionMatrices"<<endl;

 // optionally the built-in test suite, useful mainly for development
 /*
  ProductionMatrices productionMatrices;
  
 
      productionMatrices.test();
 
 */

  /////////////////////////////
  // sample example of usage //
  /////////////////////////////


  // define the energy grids and the proton, Helium spectra

  double E_proton_min = 400.; // MeV
  double E_gamma_min =   50.;
 
  double E_proton_factor=1.2; // factor for energy grid
  double E_gamma_factor =1.1;

  int nE_proton_grid =100;    // number of grid points 
  int nE_gamma_grid  = 50;


  valarray<double> E_proton_grid ;
  valarray<double> E_gamma_grid ;

  E_proton_grid .resize(nE_proton_grid );
  E_gamma_grid  .resize(nE_gamma_grid  );

  // the proton and Helium energy scales are the same, in MeV/nucleon
  // NB the energy scale must be logarithmic for the integration routines to work correctly !

  for(int i=0;i<nE_proton_grid ;i++)
   E_proton_grid [i]=  E_proton_min *  pow(E_proton_factor,i);

  // the gamma-ray energy scale need not be logarithmic, but here it is
  for(int i=0;i<nE_gamma_grid  ;i++)
   E_gamma_grid  [i]=  E_gamma_min  *  pow(E_gamma_factor ,i);

  valarray<double> proton_spectrum;
  valarray<double> helium_spectrum;
  proton_spectrum.resize(nE_proton_grid );
  helium_spectrum.resize(nE_proton_grid );

 // the units required are flux in particles/cm^2/s/sr/(MeV/nucleon), energy is kinetic energy in this example

 // define ad hoc power-law proton and Helium spectra with right order of magnitude for test
 // cf  1.94e4 (E/GeV)^-2.76;//  m-2 sr-1 s-1 GeV-1 Shikaze 2007  http://adsabs.harvard.edu/abs/2007APh....28..154S

  double Ap    = 2e-3; // cm-2 sr-1 s-1 MeV-1
  double AHe   = 0.1*Ap;
  double index = 2.8;

  proton_spectrum =  Ap * pow(E_proton_grid*1e-3, -index);
  helium_spectrum = AHe * pow(E_proton_grid*1e-3, -index);


  // now use the class to get the emissivities

  ProductionMatrices pm;                          // define an object of this class
  pm.read();                                      // read in the original Huang matrices
  pm.init("kinetic_energy","MeV");                // define the conventions to be used: can also use "total_energy" or "GeV"
                                                  //  (always per nucleon)
  pm.rebin( E_proton_grid , E_gamma_grid );       // prepare the matrix on the required grid: only needed once per grid

  pm.emissivity(proton_spectrum,helium_spectrum); // compute the emissivities for protons and Helium - for any spectra

//pm.print(); to see the whole contents: only if interested in the details of the matrices !

  // to see just the result:

  for(int i=0;i<nE_gamma_grid  ;i++)
    cout<<"sample E_gamma = " << E_gamma_grid[i]<<" MeV"
         <<" emissivity p = " << pm.gamma_decay_p_emissivity_rebin [i]
         <<           " He = "<< pm.gamma_decay_he_emissivity_rebin[i]
	 <<" photons/H atom/sr/s/MeV" 
         <<endl;
  

  // compute emissivity in an energy band
  // (for this pm.rebin is not required since done internally according to the band)

  double E_gamma_band_min=100.; // MeV
  double E_gamma_band_max=200.;
  double E_gamma_band_factor=1.01; // used to integrate over band

  double emiss_band = pm.emissivity_band(proton_spectrum, helium_spectrum, E_proton_grid, 
                                         E_gamma_band_min,E_gamma_band_max,E_gamma_band_factor);

  cout<<"sample E_gamma = "<<E_gamma_band_min<<"-"<<E_gamma_band_max<<" MeV"
      <<" emissivity  = "<<emiss_band
      <<" photons/H atom/sr/s" 
      <<endl;

  // there is also a version for multiple bands:

  valarray<double> E_gamma_band_min_(3);
  valarray<double> E_gamma_band_max_(3);
  valarray<double>       emiss_band_(3);

 E_gamma_band_min_[0]=  50.;  E_gamma_band_max_[0]= 150.; // MeV
 E_gamma_band_min_[1]=  1e3;  E_gamma_band_max_[1]= 2e3;
 E_gamma_band_min_[2]=  1e5;  E_gamma_band_max_[2]= 2e5;

 emiss_band_ = pm.emissivity_band(proton_spectrum, helium_spectrum, E_proton_grid, 
                                  E_gamma_band_min_,E_gamma_band_max_,E_gamma_band_factor);

 for(int i=0;i<3;i++)
  cout<<"sample E_gamma = "<<E_gamma_band_min_[i]<<"-"<<E_gamma_band_max_[i]<<" MeV"
      <<" emissivity in multiple bands   = "<<emiss_band_[i]
      <<" photons/H atom/sr/s" 
      <<endl;

 // there is also a fast version with pre-computed matrices, useful for model fitting. Recommended version.

 // compute the matrices
 pm.gen_gamma_band_matrices(E_proton_grid, 
                            E_gamma_band_min_,E_gamma_band_max_,E_gamma_band_factor);

 // use the matrices, now only the input CR spectra are needed since the bands have been defined already

 emiss_band_ = pm.emissivity_band (proton_spectrum, helium_spectrum);

 cout<<"sample using precomputed matrices:"<<endl;
 for(int i=0;i<3;i++)
  cout<<"sample E_gamma = "<<E_gamma_band_min_[i]<<"-"<<E_gamma_band_max_[i]<<" MeV"
      <<" emissivity in multiple bands using precomputed matrices  = "<<emiss_band_[i]
      <<" photons/H atom/sr/s" 
      <<endl;


 cout<<"   End of test_ProductionMatrices"<<endl;

return 0;
}
